#!/usr/bin/env python
# Copyright (C) 2012  Peter Hatina <phatina@redhat.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

import os
import sys

from fmci.fmci_options import FmciBasicOptions
from fmci.fmci_address import FmciIpv4Addr, FmciHostGenerator
from fmci.fmci_client_ip import FmciIpClient

class FmciIpOptions(FmciBasicOptions):
    OBJ_UNKNOWN, OBJ_ADDR, OBJ_ROUTE = range(3)
    OP_UNKNOWN, OP_SHOW, OP_SET, OP_DEL = range(4)

    def __init__(self):
        super(self.__class__, self).__init__(
            "Available actions:\n"
            "  addr show\n"
            "  addr show [interface]\n"
            "  addr set [address] dev [interface]\n"
            "  addr del dev [interface]\n")
        self.m_parser.set_usage("Usage: %prog [options] object [command] (dev [device])")
        self.m_good = True
        self.m_operation = FmciIpOptions.OP_UNKNOWN
        self.m_object = FmciIpOptions.OBJ_UNKNOWN
        self.m_device = ""
        self.m_ifaddr = ""

    @property
    def good(self):
        return self.m_good

    @property
    def object(self):
        return self.m_object

    @property
    def operation(self):
        return self.m_operation

    @property
    def ifaddr(self):
        return self.m_ifaddr

    @property
    def device(self):
        return self.m_device

    def __posOptionsPopFront(self):
        if len(self.m_pos_options):
            return self.m_pos_options.pop(0)
        return ""

    def __parseDev(self):
        arg = self.__posOptionsPopFront()
        if not arg or arg != "dev":
            self.m_good = False
            return
        self.m_device = self.__posOptionsPopFront()
        if not self.m_device:
            self.m_good = False

    def __parseOpWithIfaddr(self):
        self.m_ifaddr = self.__posOptionsPopFront()
        if not self.m_ifaddr:
           self.m_good = False
           return
        self.__parseDev()

    def __parseOpShow(self):
        self.m_operation = FmciIpOptions.OP_SHOW
        arg = self.m_pos_options[0] if len(self.m_pos_options) else ""
        if arg:
            self.__parseDev()

    def __parseOpSet(self):
        self.m_operation = FmciIpOptions.OP_SET
        self.__parseOpWithIfaddr()

    def __parseOpDel(self):
        self.m_operation = FmciIpOptions.OP_DEL
        self.__parseDev()

    def __parseAddr(self):
        self.m_object = FmciIpOptions.OBJ_ADDR
        arg = self.__posOptionsPopFront()
        if arg == "show":
            self.__parseOpShow()
        elif arg == "set":
            self.__parseOpSet()
        elif arg == "del":
            self.__parseOpDel()
        else:
            self.m_operation = FmciIpOptions.OP_UNKNOWN
            self.m_good = False

    def __parseRoute(self):
        self.m_object = FmciIpOptions.OBJ_ROUTE

    def parseCmds(self):
        arg = self.__posOptionsPopFront()
        if arg == "addr":
            self.__parseAddr()
        elif arg == "route":
            self.__parseRoute()
        else:
            self.m_good = False

if __name__ == "__main__":
    options = FmciIpOptions()
    options.parse(sys.argv)
    if not options.good:
        options.printWrongUsage()
        sys.exit(1)
    options.parseCmds()
    if not options.good:
        options.printWrongUsage()
        sys.exit(1)

    client_failed = False
    client_hostnames = FmciHostGenerator.enumerate(options.hostname)
    for client_hostname in client_hostnames:
        if not client_hostname:
            continue
        client = FmciIpClient(client_hostname.name, options.username, options.password)
        rval = True
        errorstr = ""
        if options.object == FmciIpOptions.OBJ_ADDR:
            if options.operation == FmciIpOptions.OP_SHOW:
                if options.device:
                    (rval, errorstr) = client.ipListDevices(options.device)
                else:
                    (rval, errorstr) = client.ipListDevices()
            elif options.operation == FmciIpOptions.OP_SET:
                (rval, errorstr) = client.ipSetAddress(options.device, options.ifaddr)
            elif options.operation == FmciIpOptions.OP_DEL:
                (rval, errorstr) = client.ipDelAddress(options.device)
        elif options.object == FmciIpOptions.OBJ_ROUTE:
            # TODO: implement
            sys.stderr.write("Route operation not implemented yet!\n")
            sys.exit(1)
        else:
            sys.stderr.write("Unknown object specified\n")

        if rval:
            sys.stdout.write("%s: %s\n" % (client_hostname.name, errorstr if errorstr else "ok"))
        else:
            sys.stderr.write("%s: %s\n" % (client_hostname.name, errorstr))
            client_failed = True

    sys.exit(client_failed)
