# Copyright (C) 2012  Peter Hatina <phatina@redhat.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

import re
import sys
import itertools

class FmciHost(object):
    def __init__(self, host):
        self.m_host = host

    @property
    def name(self):
        if isinstance(self.m_host, FmciIpv4Addr):
            return self.m_host.address
        elif isinstance(self.m_host, FmciIpv6Addr):
            return self.m_host.address
        else:
            return self.m_host

class FmciIpv4Addr(object):
    class FmciIpv4Member(object):
        def __init__(self, member):
            self.m_member = member

        def enumerate(self):
            match = re.search("^{(.*)}$", self.m_member)
            member = match.group(1) if match else []
            if not member:
                return [self.m_member]
            ranges = (x.split("-") for x in member.split(";"))
            try:
                members = []
                for r in ranges:
                    low = int(r[0])
                    high = int(r[-1])
                    if low > high:
                        low, high = high, low
                    if not 0 <= low <= 255 or not 0 <= high <= 255:
                        return []
                    members += [str(i) for i in range(low, high + 1)]
            except ValueError:
                return []
            return members

    def __init__(self, addr):
        unpacked = addr.split("/")
        self.m_addr = unpacked[0]
        if len(unpacked) > 1:
            self.m_mask = int(unpacked[1])
        else:
            first_octet = int(self.m_addr.split(".")[0])
            if first_octet & 0x80 == 0:
                self.m_mask = 8
            elif first_octet & 0xC0 == 0x80:
                self.m_mask = 16
            elif first_octet & 0xE0 == 0xC0:
                self.m_mask = 24

    @property
    def address(self):
        return self.m_addr

    @property
    def netmask(self):
        rval = ""
        remain = self.m_mask
        for i in range(0, 4):
            p = 8
            if remain < 8:
                p = remain
            remain -= p
            member = 0
            for j in range(8 - p, 8):
                member += pow(2, j)
            rval += str(member) + "."
        return rval[:-1]

    @staticmethod
    def enumerate(pattern):
        str_members = pattern.split(".")
        if len(str_members) != 4:
            return []
        members = []
        for m in str_members:
            members.append(FmciIpv4Addr.FmciIpv4Member(m))
        addresses = members[-1].enumerate()
        if not addresses:
            return []
        for i in reversed(range(0, len(members) - 1)):
            combined = []
            members_to_combine = members[i].enumerate()
            if not members_to_combine:
                return []
            for r in itertools.product(members_to_combine, addresses):
                combined.append(".".join([r[0], r[1]]))
                addresses = combined
        rval = []
        for addr in sorted(set(addresses)):
            rval.append(FmciHost(FmciIpv4Addr(addr)))
        return rval

class FmciIpv6Addr(object):
    def __init__(self, addr):
        self.m_addr = addr

    @property
    def address(self):
        return self.m_addr

    class FmciIpv6Member(object):
        def __init__(self, member):
            self.m_member = member

        def enumerate(self):
            match = re.search("^{(.*)}$", self.m_member)
            member = match.group(1) if match else []
            if not member:
                return [self.m_member]
            ranges = (x.split("-") for x in member.split(";"))
            try:
                members = []
                for r in ranges:
                    low = int(r[0], 16)
                    high = int(r[-1], 16)
                    if low > high:
                        low, high = high, low
                    if not 0 <= low <= 65535 or not 0 <= high <= 65535:
                        return []
                    members += [str(hex(i)[2:]) for i in range(low, high + 1)]
            except ValueError:
                return []
            return members

    @staticmethod
    def enumerate(pattern):
        if pattern.count("::") > 1 or pattern.count(":::") > 0:
            return []
        str_members = pattern.split(":")
        members = []
        for m in str_members:
            members.append(FmciIpv6Addr.FmciIpv6Member(m))
        addresses = members[-1].enumerate()
        if not addresses:
            return []
        for i in reversed(range(0, len(members) - 1)):
            combined = []
            members_to_combine = members[i].enumerate()
            if not members_to_combine:
                return []
            for r in itertools.product(members_to_combine, addresses):
                combined.append(":".join([r[0], r[1]]))
                addresses = combined
        rval = []
        for addr in sorted(set(addresses)):
            rval.append(FmciHost(FmciIpv6Addr(addr)))
        return rval

class FmciHostGenerator(object):
    PATTERN_TYPE_FQDN, \
    PATTERN_TYPE_IPV4, \
    PATTERN_TYPE_IPV6 = range(3)

    @staticmethod
    def recognizePatternType(pattern):
        pattern_regexps = (
            ("^(([\d]{1,3})|(\{([\d]{1,3}|[\d]{1,3}-[\d]{1,3})(;("
             "[\d]{1,3}|[\d]{1,3}-[\d]{1,3}))*\}))\.(([\d]{1,3})"
             "|(\{([\d]{1,3}|[\d]{1,3}-[\d]{1,3})(;([\d]{1,3}|[\d"
             "]{1,3}-[\d]{1,3}))*\}))\.(([\d]{1,3})|(\{([\d]{1,3"
             "}|[\d]{1,3}-[\d]{1,3})(;([\d]{1,3}|[\d]{1,3}-[\d]{1"
             ",3}))*\}))\.(([\d]{1,3})|(\{([\d]{1,3}|[\d]{1,3}-["
             "\d]{1,3})(;([\d]{1,3}|[\d]{1,3}-[\d]{1,3}))*\}))$",
             FmciHostGenerator.PATTERN_TYPE_IPV4),
            ("^(([\da-fA-F]{1,4})|(\{([\da-fA-F]{1,4}|[\da-fA-F]{"
             "1,4}-[\da-fA-F]{1,4})(;([\da-fA-F]{1,4}|[\da-fA-F]"
             "{1,4}-[\da-fA-F]{1,4}))*\})){0,1}(:(([\da-fA-F]{1,"
             "4})|(\{([\da-fA-F]{1,4}|[\da-fA-F]{1,4}-[\da-fA-F]"
             "{1,4})(;([\da-fA-F]{1,4}|[\da-fA-F]{1,4}-[\da-fA-F"
             "]{1,4}))*\})*)){0,6}:(([\da-fA-F]{1,4})|(\{([\da-f"
             "A-F]{1,4}|[\da-fA-F]{1,4}-[\da-fA-F]{1,4})(;([\da-f"
             "A-F]{1,4}|[\da-fA-F]{1,4}-[\da-fA-F]{1,4}))*\}))$",
             FmciHostGenerator.PATTERN_TYPE_IPV6)
        )
        for (p, rval) in pattern_regexps:
            if re.match(p, pattern):
                return rval
        return FmciHostGenerator.PATTERN_TYPE_FQDN

    @staticmethod
    def enumerate(lst):
        rval = []
        for pattern in lst.split(","):
            pattern_type = FmciHostGenerator.recognizePatternType(pattern)
            if pattern_type == FmciHostGenerator.PATTERN_TYPE_FQDN:
                rval.append(FmciHost(pattern))
            elif pattern_type == FmciHostGenerator.PATTERN_TYPE_IPV4:
                rval.extend(FmciIpv4Addr.enumerate(pattern))
            elif pattern_type == FmciHostGenerator.PATTERN_TYPE_IPV6:
                rval.extend(FmciIpv6Addr.enumerate(pattern))
            else:
                sys.stderr.write("Unrecognized host pattern supplied!\n")
                return []
        return rval
