# Copyright (C) 2012  Peter Hatina <phatina@redhat.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

import sys
import pywbem
from fmci_client_base import FmciBaseClient
from fmci_address import FmciIpv4Addr

class FmciIpClient(FmciBaseClient):
    LMI_ETHERNET_PORT_CLASS_NAME = "LMI_EthernetPort"
    LMI_IP_ASSIGNMENT_SETTING_DATA_CLASS_NAME = "LMI_IPAssignmentSettingData"
    IP_CONFIGURATION_SERVICE_CLASS_NAME = "LMI_IPConfigurationService"
    IP_PROTOCOL_ENDPOINT_CLASS_NAME = "LMI_IPProtocolEndpoint"
    IP_PROTOCOL_TYPE_IPV4 = 1
    IP_PROTOCOL_TYPE_IPV6 = 2

    def __init__(self, hostname, username = "", password = ""):
        super(self.__class__, self).__init__(hostname, username, password)

    def __ipGetIpProtocolEndpointInstances(self, device = ""):
        (inst_list, errorstr) = self._getInstances(FmciIpClient.IP_PROTOCOL_ENDPOINT_CLASS_NAME)
        if not inst_list:
            return (None, errorstr)
        rval_inst_list = filter(lambda n: device == n["Name"].split("_")[0], inst_list) if device else inst_list
        if not rval_inst_list:
            return (None, "No such device '%s' found" % device)
        return (rval_inst_list if len(rval_inst_list) > 1 else rval_inst_list[0], "")

    def __ipPrintDeviceInfo(self, device):
        device_list = device if type(device) == list else [device]
        for d in device_list:
            sys.stdout.write("%s:\n" % d["Name"])
            sys.stdout.write("    inet: %s\n" % d["IPv4Address"])
            sys.stdout.write("    inet6: %s\n" % d["IPv6Address"])

    def __ipGetAddressType(self, address):
        # blind ipv4 check
        prot_type = FmciIpClient.IP_PROTOCOL_TYPE_IPV6
        if address.count(".") == 3:
            prot_type = FmciIpClient.IP_PROTOCOL_TYPE_IPV4
        return prot_type

    def __ipGetEthernetPortInstance(self, device):
        return self._getInstances(FmciIpClient.LMI_ETHERNET_PORT_CLASS_NAME, device, Key = "DeviceID")

    def __ipGetIpAssignmentSettingData(self, device):
        (inst_list, errorstr) = self._getInstances(FmciIpClient.LMI_IP_ASSIGNMENT_SETTING_DATA_CLASS_NAME)
        if not inst_list:
            return (None, errorstr)
        inst_list = inst_list if type(inst_list) == list else [inst_list]
        rval_inst_list = filter(lambda x: x["Caption"] == "static-" + device, inst_list)
        if not rval_inst_list:
            return (None, "No IP setting for device %s found" % device)
        return (rval_inst_list, "")

    def __ipRemoveStaticSettings(self, device):
        (inst_list, errorstr) = self.__ipGetIpAssignmentSettingData(device)
        if not inst_list:
            return (False, errorstr)
        for inst in inst_list:
            (rval, errorstr) = self._deleteInstance(inst)
            if not rval:
                return (False, errorstr)
        return (True, "")

    def __ipCreateStaticSetting(self, device, address, **kwargs):
        self.__ipRemoveStaticSettings(device)
        (device_inst, errorstr) = self.__ipGetEthernetPortInstance(device)
        if not device_inst:
            return (False, {}, errorstr)
        prot_type = self.__ipGetAddressType(address)
        args = {
            "Caption" : "static-" + device,
            "ProtocolIFType" : pywbem.Uint16(prot_type),
            "Port" : device_inst.path
        }
        if prot_type == FmciIpClient.IP_PROTOCOL_TYPE_IPV4:
            args["IPv4Address"] = FmciIpv4Addr(address).address
            if "SubnetMask" in kwargs:
                args["IPv4SubnetMask"] = kwargs["SubnetMask"]
            else:
                args["IPv4SubnetMask"] = FmciIpv4Addr(address).netmask
            if "DefaultGateway" in kwargs:
                args["IPv4DefaultGateway"] = kwargs["DefaultGateway"]
            if "DNSServers" in kwargs:
                args["IPv4DNSServers"] = kwargs["DNSServers"]
            if "SearchDomains" in kwargs:
                args["IPv4SearchDomains"] = kwargs["SearchDomains"]
        else:
            args["IPv6Address"] = address
            if "DefaultGateway" in kwargs:
                args["IPv6DefaultGateway"] = kwargs["DefaultGateway"]
            if "DNSServers" in kwargs:
                args["IPv6DNSServers"] = kwargs["DNSServers"]
            if "SearchDomains" in kwargs:
                args["IPv6SearchDomains"] = kwargs["SearchDomains"]
            if "PrefixLength" in kwargs:
                args["IPv6PrefixLength"] = pywbem.Uint8(kwargs["PrefixLength"])
        (configuration_inst, errorstr) = self._getInstances(FmciIpClient.IP_CONFIGURATION_SERVICE_CLASS_NAME)
        if not configuration_inst:
            return (False, {}, errorstr)
        return self._callMethod(configuration_inst, "CreateStaticSetting", **args)

    def __ipApplyStaticSetting(self, device, setting_inst):
        (configuration_inst, errorstr) = self._getInstances(FmciIpClient.IP_CONFIGURATION_SERVICE_CLASS_NAME)
        if not configuration_inst:
            return (False, errorstr)
        (device_inst, errorstr) = self.__ipGetIpProtocolEndpointInstances(device)
        if type(device_inst) == list:
            device_inst = device_inst[0]
        if not device_inst:
            return (False, errorstr)
        args = {
            "Configuration": setting_inst.path,
            "Endpoint" : device_inst.path
        }
        (rval, rparam, errorstr) = self._callMethod(configuration_inst, "ApplySettingToIPProtocolEndpoint", **args)
        return (rval, errorstr)

    def ipListDevices(self, device = ""):
        (inst_list, errorstr) = self.__ipGetIpProtocolEndpointInstances(device)
        if not inst_list:
            return (False, errorstr)
        self.__ipPrintDeviceInfo(inst_list)
        return (True, "")

    def ipSetAddress(self, device, address, **kwargs):
        (rval, rparam, errorstr) = self.__ipCreateStaticSetting(device, address, **kwargs)
        if not rval:
            return (False, errorstr)
        setting_inst = self.m_cliconn.GetInstance(rparam["SettingData"], LocalOnly = False)
        return self.__ipApplyStaticSetting(device, setting_inst)

    def ipDelAddress(self, device):
        return self.__ipRemoveStaticSettings(device)
